#*******************************************************************************
# Copyright (C) 2013 ID | SIS ETH Zuerich and Universitaet Basel
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#  
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#*******************************************************************************
#!/bin/bash
#
# Control script for BEE on Unix / Linux systems
#
# resolve links to our executable script
SELF_NAME="$0"
if test -e "${SELF_NAME}" -a -L "${SELF_NAME}" ; then
    SELF_NAME=$(find "${SELF_NAME}" -printf "%l")
fi
SELF_DIRNAME=$(cd "${SELF_NAME%/*}" && echo "${PWD}")

# we want to execute this script in the context of the install directory,
# otherwise we have experienced small problems starting the jar
cd "${SELF_DIRNAME}" || exit 1

###############################################################################
# Configuration values: defined in a config file, if it exists. Otherwise, 
# the config values need to be specified here
###############################################################################
# The shell configuration file that is used:
SHELLCONFFILE="${SELF_DIRNAME}/etc/bee.config"
# The system configuration file that is used:
SYSCONFFILE="${SELF_DIRNAME}/etc/system.config"


# If no configuration file is used, set following variables:
#
# Home directory of the JRE that should be used
#JAVA_HOME=${JAVA_HOME:=/usr/java/latest}
#
# Options to the JRE
#JAVA_OPTS=${JAVA_OPTS:=-Dforce-accept-ssl-certificate=true}
#
# First path where to search for libraries
# ATTENTION: it depends on the system (32, 64 bits, etc..)
#LD_LIBRARY_PATH=
#

###############################################################################
# Source shell configuration script, if any
###############################################################################
test -r "${SHELLCONFFILE}" && source "${SHELLCONFFILE}"

###############################################################################
# Parse system configuration script, if any
###############################################################################
# The port number where the REST service interface is running
export REST_SERVICE_PORT=$(grep "^system.rest.port.*=" "${SYSCONFFILE}" 2>/dev/zero|perl -pe 's/.*=\s*([0-9]+).*/$1/g')

###############################################################################
# Set other variables
###############################################################################
# startup log file to create
STARTUPLOG="${SELF_DIRNAME}/log/startup_log.txt"

# Process ID file to create
PIDFILE="${SELF_DIRNAME}/bee.pid"

# Java binary 
if test -n "${JAVA_HOME}" ; then
  JAVA_BIN="${JAVA_HOME}/bin/java"
else
  JAVA_BIN="java"
fi

# Max number of loops to wait for initial self-test to finish
MAX_LOOPS=10

###############################################################################
# Method definitions
###############################################################################
isPIDRunning()
{
  if [ "$1" = "" ]; then
    return 1
  fi
  if [ "$1" = "fake" ]; then # for unit tests
    return 0
  fi
  # This will have a return value of 0 on BSDish systems
  isBSD=$(ps aux >/dev/null 2>&1; echo $?)
  if [ "$isBSD" == "0" ]; then
    if [ "$(ps aux | awk -v PID=$1 '{if ($2==PID) {print "FOUND"}}')" = "FOUND" ]; then    	
      return 0
    else
      return 1
    fi
  else
    if [ "$(ps -ef | awk -v PID=$1 '{if ($2==PID) {print "FOUND"}}')" = "FOUND" ]; then
      return 0
    else
      return 1
    fi
  fi
}

getPidStatus()
{
  if [ -f "${PIDFILE}" ]; then
    PID=$(cat ${PIDFILE} 2>/dev/null)
    isPIDRunning ${PID}
# To be implemented: status SHUTDOWN, ERROR, PROCESSING    
    if [ $? -eq 0 ]; then
#      if [ -f .MARKER_shutdown ]; then
#        STATUS=SHUTDOWN
#        return 2
#      elif [ "`ls -a1 .MARKER_*_error 2>/dev/null`" != "" ]; then
#        STATUS=ERROR
#        return 1
#      elif [ "`ls -a1 .MARKER_*_processing 2>/dev/null`" != "" ]; then
#        STATUS=PROCESSING
#      else
#        STATUS=IDLE
        STATUS="UNKNOWN"
#      fi
      return 0
    else
      STATUS="STALE"
      return 4
    fi
  else
    STATUS="DOWN"
    return 3
  fi
}

getFullStatus()
{
  # check the PID first:
  getPidStatus
  # if the PID exists as a running process, then check the REST status:
  if test $? -le 3; then
    COUNTRUNNINGPROCESSES=$(/usr/bin/curl --request GET "http://localhost:${REST_SERVICE_PORT}/apiv1/processes/countRunningProcesses" 2>/dev/null)
    if test $? -ne 0; then
      STATUS="DOWN"
      return 3
    else
      if test $COUNTRUNNINGPROCESSES -le 0; then
        STATUS="IDLE"
      else
        STATUS="PROCESSING"
      fi
      return 0
    fi
  fi
}

printStatus()
{
  PID=$(cat "${PIDFILE}" 2>/dev/null)
  if test -z ${PID}; then
    PID="unknown"
  fi
  MSG_PREFIX="Bee (pid ${PID}) is"
  case "$1" in
    ERROR)
      echo "${MSG_PREFIX} running and in error state:"
      cat .MARKER_*_error 2>/dev/null
      echo
      ;;
    PROCESSING)
      echo "${MSG_PREFIX} running and in processing state, with $COUNTRUNNINGPROCESSES process(es) running"
      ;;
    IDLE)
      echo "${MSG_PREFIX} running and in idle state"
      ;;
    UNKNOWN)
      echo "${MSG_PREFIX} running but in an unknown state (request not implemented)"
      ;;
    SHUTDOWN)
      echo "${MSG_PREFIX} in shutdown mode"
      ;;
    DOWN)
      echo "Bee is not running"
      ;;
    STALE)
      echo "Bee is dead (stale pid ${PID})"
      ;;
  esac
}

###############################################################################
# Execute shell script command
###############################################################################
command=$1
# ensure that we ignore a possible prefix "--" for any command 
command="${command#--*}"

case "$command" in
  start)
    getPidStatus
    EXIT_STATUS=$?

    if [ ${EXIT_STATUS} -lt 3 ]; then
      echo "Cannot start Bee:"
      printStatus "${STATUS}"
      exit 100
    fi
	mkdir -p log
    echo -n "Starting Bee "
    shift 1
    nice -n "19" "${JAVA_BIN}" ${JAVA_OPTS} -jar -Dlog4j.configuration="file:${SELF_DIRNAME}/etc/log4j.xml" -Dlogback.configurationFile="${SELF_DIRNAME}/etc/logback.xml" "${SELF_DIRNAME}/bee-workflowmanager.jar" "$@" > ${STARTUPLOG} 2>&1 & echo $! > ${PIDFILE}

    if [ $? -eq 0 ]; then
      # wait for initial self-test to finish
      n=0
      while [ $n -lt ${MAX_LOOPS} ]; do
        sleep 1
        if [ ! -f ${PIDFILE} ]; then
          break
        fi
        if [ -s "${STARTUPLOG}" ]; then
          PID=$(cat ${PIDFILE} 2>/dev/null)
          isPIDRunning ${PID}
          if [ $? -ne 0 ]; then
            break
          fi
        fi
        n=$(($n+1))
      done 
      PID=$(cat "${PIDFILE}" 2>/dev/null)
      isPIDRunning ${PID}
      if [ $? -eq 0 ]; then
        echo "(pid ${PID})"
      else
        echo "FAILED"
        echo "startup log says:"
        cat "${STARTUPLOG}"
      fi
    else
      echo "FAILED"
    fi
  ;;

  stop)
    echo -n "Stopping Bee "
    if [ -f "${PIDFILE}" ]; then
      PID=$(cat "${PIDFILE}" 2>/dev/null)
      isPIDRunning ${PID}
      if [ $? -eq 0 ]; then
        kill ${PID}
        n=0
        while [ $n -lt ${MAX_LOOPS} ]; do
          isPIDRunning ${PID}
          if [ $? -ne 0 ]; then
            break
          fi
          sleep 1
          n=$(($n+1))
        done
        isPIDRunning ${PID}
        if [ $? -ne 0 ]; then
          echo "(pid ${PID})"
          test -f "${PIDFILE}" && rm "${PIDFILE}" 2>/dev/null
        else
          echo "FAILED"
        fi
      else
        if [ -f "${PIDFILE}" ]; then
          rm "${PIDFILE}" 2>/dev/null
          echo "(was dead - cleaned up pid file)"
        fi
      fi
    else
      echo "(not running - nothing to do)"
    fi
  ;;

  restart)
    ${SELF_NAME} stop
    ${SELF_NAME} start
  ;;

# To be implemented  
#  shutdown)
#    touch $MARKER_SHUTDOWN_FILE
#  ;;

  status)
    getFullStatus
    EXIT_STATUS=$?
    printStatus "${STATUS}"
    exit ${EXIT_STATUS}
  ;;

  mstatus)
    getFullStatus
    EXIT_STATUS=$?
    echo "${STATUS}"
    exit ${EXIT_STATUS}
  ;;

# To be implemented
#  recover)
#    echo "Triggering recovery cycle"
#    touch .MARKER_recovery
#  ;;
#  help)
#    ${JAVA_BIN} ${JAVA_OPTS} -jar lib/ibrain2.jar --help
#  ;;
#  version)
#    ${JAVA_BIN} ${JAVA_OPTS} -jar lib/ibrain2.jar --version
#  ;;
#  test-notify)
#    ${JAVA_BIN} ${JAVA_OPTS} -jar lib/ibrain2.jar --test-notify
#  ;;

  *)
    echo "Usage: $0 {start|stop|restart|status|mstatus}"
    exit 200
esac
exit 0
